@@ -696,12 +696,13 @@ static int object_find_next (
                 object_instance = list_entry (list, struct object_instance,
 			child_list);
 
-		if ((object_instance->object_name_len ==
-			object_find_instance->object_len) &&
+		if (object_find_instance->object_len == 0 ||
+		    ((object_instance->object_name_len ==
+		      object_find_instance->object_len) &&
 
-			(memcmp (object_instance->object_name,
-				object_find_instance->object_name,
-				object_find_instance->object_len) == 0)) {
+		     (memcmp (object_instance->object_name,
+			      object_find_instance->object_name,
+			      object_find_instance->object_len) == 0))) {
 
 			found = 1;
 			break;
@@ -771,6 +772,92 @@ error_exit:
 	return (-1);
 }
 
+static int object_key_increment (
+	unsigned int object_handle,
+	void *key_name,
+	int key_len,
+	unsigned int *value)
+{
+	unsigned int res = 0;
+	struct object_instance *instance;
+	struct object_key *object_key = NULL;
+	struct list_head *list;
+	int found = 0;
+
+	res = hdb_handle_get (&object_instance_database,
+		object_handle, (void *)&instance);
+	if (res != 0) {
+		goto error_exit;
+	}
+	for (list = instance->key_head.next;
+		list != &instance->key_head; list = list->next) {
+
+		object_key = list_entry (list, struct object_key, list);
+
+		if ((object_key->key_len == key_len) &&
+			(memcmp (object_key->key_name, key_name, key_len) == 0)) {
+			found = 1;
+			break;
+		}
+	}
+	if (found && object_key->value_len == sizeof(int)) {
+		(*(int *)object_key->value)++;
+		*value = *(int *)object_key->value;
+	}
+	else {
+		res = -1;
+	}
+
+	hdb_handle_put (&object_instance_database, object_handle);
+	return (res);
+
+error_exit:
+	return (-1);
+}
+
+static int object_key_decrement (
+	unsigned int object_handle,
+	void *key_name,
+	int key_len,
+	unsigned int *value)
+{
+	unsigned int res = 0;
+	struct object_instance *instance;
+	struct object_key *object_key = NULL;
+	struct list_head *list;
+	int found = 0;
+
+	res = hdb_handle_get (&object_instance_database,
+		object_handle, (void *)&instance);
+	if (res != 0) {
+		goto error_exit;
+	}
+	for (list = instance->key_head.next;
+		list != &instance->key_head; list = list->next) {
+
+		object_key = list_entry (list, struct object_key, list);
+
+		if ((object_key->key_len == key_len) &&
+			(memcmp (object_key->key_name, key_name, key_len) == 0)) {
+			found = 1;
+			break;
+		}
+	}
+	if (found && object_key->value_len == sizeof(int)) {
+		(*(int *)object_key->value)--;
+		*value = *(int *)object_key->value;
+	}
+	else {
+		res = -1;
+	}
+
+	hdb_handle_put (&object_instance_database, object_handle);
+	return (res);
+
+error_exit:
+	return (-1);
+}
+
 static int object_key_delete (
 	unsigned int object_handle,
 	void *key_name,
@@ -1000,14 +1087,13 @@ static int _dump_object(struct object_instance *instance, FILE *file, int depth)
 	return 0;
 }
 
-
 static int object_key_iter_reset(unsigned int object_handle)
 {
 	unsigned int res;
 	struct object_instance *instance;
 
 	res = hdb_handle_get (&object_instance_database,
-		object_handle, (void *)&instance);
+			      object_handle, (void *)&instance);
 	if (res != 0) {
 		goto error_exit;
 	}
@@ -1020,6 +1106,7 @@ error_exit:
 	return (-1);
 }
 
+
 static int object_key_iter(unsigned int parent_object_handle,
 			   void **key_name,
 			   int *key_len,
@@ -1064,163 +1151,6 @@ error_exit:
 	return (-1);
 }
 
-static int object_iter_reset(unsigned int parent_object_handle)
-{
-	unsigned int res;
-	struct object_instance *instance;
-
-	res = hdb_handle_get (&object_instance_database,
-		parent_object_handle, (void *)&instance);
-	if (res != 0) {
-		goto error_exit;
-	}
-	instance->iter_list = &instance->child_head;
-
-	hdb_handle_put (&object_instance_database, parent_object_handle);
-	return (0);
-
-error_exit:
-	return (-1);
-}
-
-static int object_iter(unsigned int parent_object_handle,
-		       void **object_name,
-		       int *name_len,
-		       unsigned int *object_handle)
-{
-	unsigned int res;
-	struct object_instance *instance;
-	struct object_instance *find_instance = NULL;
-	struct list_head *list;
-	unsigned int found = 0;
-
-	res = hdb_handle_get (&object_instance_database,
-		parent_object_handle, (void *)&instance);
-	if (res != 0) {
-		goto error_exit;
-	}
-	res = -ENOENT;
-	list = instance->iter_list->next;
-	if (list != &instance->child_head) {
-
-                find_instance = list_entry (list, struct object_instance,
-					    child_list);
-		found = 1;
-	}
-	instance->iter_list = list;
-
-	if (found) {
-		*object_handle = find_instance->object_handle;
-		*object_name = find_instance->object_name;
-		*name_len = find_instance->object_name_len;
-		res = 0;
-	}
-	else {
-		res = -1;
-	}
-
-	return (res);
-
-error_exit:
-	return (-1);
-}
-
-
-static int object_find_from(unsigned int parent_object_handle,
-			    unsigned int start_pos,
-			    void *object_name,
-			    int object_name_len,
-			    unsigned int *object_handle,
-			    unsigned int *next_pos)
-{
-	unsigned int res;
-	unsigned int pos = 0;
-	struct object_instance *instance;
-	struct object_instance *find_instance = NULL;
-	struct list_head *list;
-	unsigned int found = 0;
-
-	res = hdb_handle_get (&object_instance_database,
-		parent_object_handle, (void *)&instance);
-	if (res != 0) {
-		goto error_exit;
-	}
-	res = -ENOENT;
-	for (list = instance->child_head.next;
-		list != &instance->child_head; list = list->next) {
-
-                find_instance = list_entry (list, struct object_instance,
-			child_list);
-
-		if ((find_instance->object_name_len == object_name_len) &&
-			(memcmp (find_instance->object_name, object_name,
-			object_name_len) == 0)) {
-			if (pos++ == start_pos) {
-				found = 1;
-				break;
-			}
-		}
-	}
-
-	hdb_handle_put (&object_instance_database, parent_object_handle);
-	if (found) {
-		*object_handle = find_instance->object_handle;
-		res = 0;
-	}
-	*next_pos = pos;
-	return (res);
-
-error_exit:
-	return (-1);
-}
-
-static int object_iter_from(unsigned int parent_object_handle,
-			    unsigned int start_pos,
-			    void **object_name,
-			    int *name_len,
-			    unsigned int *object_handle)
-{
-	unsigned int res;
-	unsigned int pos = 0;
-	struct object_instance *instance;
-	struct object_instance *find_instance = NULL;
-	struct list_head *list;
-	unsigned int found = 0;
-
-	res = hdb_handle_get (&object_instance_database,
-		parent_object_handle, (void *)&instance);
-	if (res != 0) {
-		goto error_exit;
-	}
-	res = -ENOENT;
-
-	for (list = instance->child_head.next;
-		list != &instance->child_head; list = list->next) {
-
-                find_instance = list_entry (list, struct object_instance,
-					    child_list);
-		if (pos++ == start_pos) {
-			found = 1;
-			break;
-		}
-	}
-
-	if (found) {
-		*object_handle = find_instance->object_handle;
-		*object_name = find_instance->object_name;
-		*name_len = find_instance->object_name_len;
-		res = 0;
-	}
-	else {
-		res = -1;
-	}
-
-	return (res);
-
-error_exit:
-	return (-1);
-}
-
 static int object_key_iter_from(unsigned int parent_object_handle,
 				unsigned int start_pos,
 				void **key_name,
@@ -1296,6 +1226,27 @@ static int object_parent_get(unsigned int object_handle,
 	return (0);
 }
 
+static int object_name_get(unsigned int object_handle,
+			   char *object_name,
+			   int *object_name_len)
+{
+	struct object_instance *instance;
+	unsigned int res;
+
+	res = hdb_handle_get (&object_instance_database,
+			      object_handle, (void *)&instance);
+	if (res != 0) {
+		return (res);
+	}
+
+	memcpy(object_name, instance->object_name, instance->object_name_len);
+	*object_name_len = instance->object_name_len;
+
+	hdb_handle_put (&object_instance_database, object_handle);
+
+	return (0);
+}
+
 
 static int object_track_start(unsigned int object_handle,
 							  object_track_depth_t depth,
@@ -1449,21 +1400,20 @@ struct objdb_iface_ver0 objdb_iface = {
 	.object_find_create	= object_find_create,
 	.object_find_next	= object_find_next,
 	.object_find_destroy	= object_find_destroy,
-	.object_find_from	= object_find_from,
 	.object_key_get		= object_key_get,
-	.object_key_iter	= object_key_iter,
 	.object_key_iter_reset	= object_key_iter_reset,
+	.object_key_iter	= object_key_iter,
 	.object_key_iter_from	= object_key_iter_from,
-	.object_iter	        = object_iter,
-	.object_iter_reset	= object_iter_reset,
-	.object_iter_from	= object_iter_from,
 	.object_priv_get	= object_priv_get,
 	.object_parent_get	= object_parent_get,
+	.object_name_get	= object_name_get,
 	.object_track_start	= object_track_start,
 	.object_track_stop	= object_track_stop,
 	.object_dump	        = object_dump,
 	.object_write_config    = object_write_config,
 	.object_reload_config   = object_reload_config,
+	.object_key_increment   = object_key_increment,
+	.object_key_decrement   = object_key_decrement,
 };
 
 struct lcr_iface objdb_iface_ver0[1] = {
